# export the API here
from .api import trace, get_current_trace, stack_trace, Trace

from .config import configure
from .thread import local  # XXX remove me from here


__version__ = "0.8.2"

__all__ = ["trace", "Trace", "get_current_trace", "stack_trace", "configure", "local"]
